
/*************************************************************************
------------------C# FILE-------------------------------------------------
------This File contains differents functions which allow to communicate--
------with the device-----------------------------------------------------
------This functions are a reunion of differents HPGL instructions that we
------found in the manufacturer documentation.----------------------------
-------------------------------------by AUDUREAU YOAN---------------------
-------------------------------------& DURAND ANTOINE---------------------
*************************************************************************/

#include <stdio.h>
#include <sys/io.h>
#include "CNC_MODE.h"
#include "CNC.h"
#include "UART.h"

#define d_MAX 32767
#define a_MAX 90



unsigned char CNC_OUTPUT_ABSOLUTE_COMMANDED_POSITION(int PORT, int *X, int *Y, int *Z )
{
    unsigned char Send[6]={0x40,0x4F,0x43,0x3B,0x0D,0x1A}; // Table to Send
    unsigned char Receive[32];                             //Table to receive
    while(!(UART_GET_CTS(PORT)));                          //Wait CTS changes to 1
    UART_Send_Buffer (PORT,Send,6);                        //Send the Table


    UART_Read_Buffer(PORT,Receive,32);                      //Read the answer
    if(Receive[0]!=0x58) return -1;     // if the 3 first characters  ...
    if(Receive[1]!=0x3A) return -1;     // not corresponding,...
    if(Receive[2]!=0x20) return -1;     // exit.
    *X= (Receive[3]-'0') * 100000 +     // Fill X
        (Receive[4]-'0') * 10000 +
        (Receive[5]-'0') * 1000 +     // with these 6 characters
        (Receive[6]-'0') * 100 +
        (Receive[7]-'0') * 10 +
        (Receive[8]-'0');
    if(Receive[9]!=0x3B) return -1;     // if these 4 charcters
    if(Receive[10]!=0x59) return -1;    // not corresponding
    if(Receive[11]!=0x3A) return -1;    //...
    if(Receive[12]!=0x20) return -1;    // exit
    *Y= (Receive[13]-'0') * 100000 +    // Fill Y
        (Receive[14]-'0') * 10000 +
        (Receive[15]-'0') * 1000 +      // with these 6 characters
        (Receive[16]-'0') * 100 +
        (Receive[17]-'0') * 10 +
        (Receive[18]-'0');
    if(Receive[19]!=0x3B) return -1;    // if these 4 charcters
    if(Receive[20]!=0x5A) return -1;    // not corresponding
    if(Receive[21]!=0x3A) return -1;    //...
    if(Receive[22]!=0x20) return -1;    // exit
    *Z= (Receive[23]-'0') * 100000 +
        (Receive[24]-'0') * 10000 + // Fill Z
        (Receive[25]-'0') * 1000 +
        (Receive[26]-'0') * 100 +
        (Receive[27]-'0') * 10 +    // with these 6 characters
        (Receive[28]-'0');
    if(Receive[29]!=0x3B) return -1;    // if the 3 last characters  ...
    if(Receive[30]!=0x0D) return -1;    // not corresponding...
    if(Receive[31]!=0x1A) return -1;    // exit
    return 0;
}


unsigned char CNC_OUTPUT_COMMANDED_POSITION(int PORT, int *X, int *Y, int *Z)
{
    unsigned char Send[5]={0x4F,0x43,0x3B,0x0D,0x1A};   // Table to Send

    unsigned char Receive[32];                          //Table to receive
    while(!(UART_GET_CTS(PORT)));
    UART_Send_Buffer (PORT,Send,5);                     // Send the table

    while(!(UART_Empty_Data_Holding_Register(PORT)));   //wait that this register is empty
    UART_Read_Buffer(PORT,Receive,32);                  //read the port
    if(Receive[0]!=0x58) return -1; // If these character s
    if(Receive[1]!=0x3A) return -1; // not corresponding
    if(Receive[2]!=0x20) return -1; // exit

    *X= (Receive[4]-'0') * 10000 +  // Fill X
        (Receive[5]-'0') * 1000 +
        (Receive[6]-'0') * 100 +    // Whith these 5 characters
        (Receive[7]-'0') * 10 +
        (Receive[8]-'0');
    if(Receive[9]!=0x3B) return -1;
    if(Receive[10]!=0x59) return -1;    // if these characters
    if(Receive[11]!=0x3A) return -1;    // not corresponding
    if(Receive[12]!=0x20) return -1;    // exit

    *Y= (Receive[14]-'0') * 10000 +     // Fill Y
        (Receive[15]-'0') * 1000 +
        (Receive[16]-'0') * 100 +       // with these 5 characters
        (Receive[17]-'0') * 10 +
        (Receive[18]-'0');
    if(Receive[19]!=0x3B) return -1;    //if these characters
    if(Receive[20]!=0x5A) return -1;
    if(Receive[21]!=0x3A) return -1;    // not corresponding
    if(Receive[22]!=0x20) return -1;    // exit

    *Z=
        (Receive[24]-'0') * 10000 + //Fill Z
        (Receive[25]-'0') * 1000 +
        (Receive[26]-'0') * 100 +   // with these 5 characters
        (Receive[27]-'0') * 10 +
        (Receive[28]-'0');
    if(Receive[29]!=0x3B) return -1;    // if these 3 last
    if(Receive[30]!=0x0D) return -1;    // characters not corresponding
    if(Receive[31]!=0x1A) return -1;    // exit

    return 0;
}


unsigned char CNC_CONTOURING_SEGMENT_LENGTH(int PORT, int d)
{
    unsigned char Send[13]={'@','C','L',0x20,0x30,0x30,0x30,0x30,0x30,0x3B,0x0D,0x1A}; //table to send
    if(d<=d_MAX)    // if d<=d_MAX
    {
        Send[4]=(d/10000)%10+'0';   // insert the value of "d" into the table Send[]
        Send[5]=(d/1000)%10+'0';
        Send[6]=(d/100)%10+'0';
        Send[7]=(d/10)%10+'0';
        Send[8]=(d%10)+'0';
    }
    else return 1;  // else exit
    while(!(UART_GET_CTS(PORT)));   // wait the bit CTS changes to 0
    UART_Send_Buffer (PORT,Send,13);    // Send the table to the device
    while((UART_GET_CTS(PORT)));    // wait the bit CTS changes to 1
    return 0;
}


unsigned char CNC_CONTOURING_ANGLE(int PORT, int a)
{
    unsigned char Send[9]={'@','C','A',0x20,0x30,0x30,0x3B,0x0D,0x1A};  // table to send
    if(a<=a_MAX)    //if a<=a_MAX
    {
        Send[4]=(a/10)%10+'0';  //insert a into the table Send
        Send[5]=(a)%10+'0';
    }
    else return 1;  // esle exit
    while(!(UART_GET_CTS(PORT)));   // wait the bit CTS changes to 0
    UART_Send_Buffer (PORT,Send,9); // Send the table to the device
    while((UART_GET_CTS(PORT)));    // wait the bit CTS changes to 1
    return 0;
}

unsigned char CNC_OUTPUT_SCALE_FACTORS(int PORT, int *X, int *Y, int *Z)
{
    unsigned char Send[5]={0x4F,0x46,0x3B,0x0D,0x1A};   //table to send
    unsigned char Receive[22];                          // table to receive
    while(!(UART_GET_CTS(PORT)));                       // wait the bit CTS changes to 0 (wait the device is not busy)
    UART_Send_Buffer (PORT,Send,5);     // Send the table
    UART_Read_Buffer(PORT,Receive,22);  // Read the port
    if(Receive[0]!=0x58) return -1;
    if(Receive[1]!=0x3A) return -1;
    *X= (Receive[2]-'0') * 100 +
        (Receive[3]-'0') * 10 +
        (Receive[4]-'0');
    if(Receive[5]!=0x3B) return -1;
    if(Receive[6]!=0x59) return -1;
    if(Receive[7]!=0x3A) return -1;
    *Y= (Receive[8]-'0') * 100 +
        (Receive[9]-'0') * 10 +
        (Receive[10]-'0');
    if(Receive[11]!=0x3B) return -1;
    if(Receive[12]!=0x5A) return -1;
    if(Receive[13]!=0x3A) return -1;
    *Z= (Receive[14]-'0') * 100 +
        (Receive[15]-'0') * 10 +
        (Receive[16]-'0');
    if(Receive[17]!=0x3B) return -1;
    if(Receive[18]!=0x20) return -1;
    if(Receive[19]!=0x20) return -1;
    if(Receive[20]!=0x0D) return -1;
    if(Receive[21]!=0x1A) return -1;
    return 0;
}



unsigned char CNC_OUTPUT_BUFFER_TOTAL_BYTE_COUNT(int PORT )
{
    int i;
    unsigned char Send[6]={0x1B,0x42,0x54,0x3A,0x0D,0x1A};  //table to send
    unsigned char Receive[18];                              //table to receive
    unsigned char Test[18]={0x1B,0x42,0x54,0x3A,0x20,0x30,0x30,0x30,0x30,0x39,0x39,0x39,0x34,0x32,0x34,0x20,0x0D,0x1A};
    while(!(UART_GET_CTS(PORT)));    // wait the bit CTS changes to 0 (wait the device is not busy)
    UART_Send_Buffer (PORT,Send,6); // send the table
    UART_Read_Buffer(PORT,Receive,18);  // read the port
    for(i=0;i<18;i++)                   // for the first Character in the table "Test" at last
        if(Receive[i]!=Test[i])     return -1;  // if these two charcater aren't identical, exit
    return 0;
}



/** Fonction d'initialisation de la machine à commande numérique */
// Valeurs retournées :
// -1 : erreur lors de la demande d'accés au port
unsigned char Initialisation(int PORT,int d)
{

    if (Open_Port(PORT)) return -1;                         // Open the serial port ...
    Config_Port (   PORT,                                   //  ... configure it ...
                    UART_38400_BAUDS,                       //  with 38400 Bauds speed
                    UART_NO_PARITY ,                        //  no parity
                    UART_ON_STOP_BIT,                       //  one stop bit
                    UART_8_BITS);                           // word length : 8 bits
    Set_DTR(PORT);      // DTR ON
    Set_RTS(PORT);      // RTS ON


    return 0;                // No error, initilization is good
}







