/*************************************************************************
------------------C# FILE-------------------------------------------------
------This File contains differents functions which allow to change-------
------the device mode-----------------------------------------------------
------They functions is a reunion of differents HPGL instructions that we-
------found in the manufacturer documentation.----------------------------
-------------------------------------by AUDUREAU YOAN---------------------
-------------------------------------& DURAND ANTOINE---------------------
*************************************************************************/

#include "CNC_MODE.h"
#include "CNC.h"
#include "UART.h"

//****************** allow to change mode to mode 0******************//

unsigned char CNC_RETURN_LOCAL_MODE(int PORT )
{
    unsigned char Send[6]={0x1B,'L','O',':',0x0D,0x1A};  // ESC L O :  CR SUB
    unsigned char Receive[6];
    while(!(UART_GET_CTS(PORT)));                           // Wait that the device isn't busy
    UART_Send_Buffer (PORT,Send,6);                         // Send the table
    UART_Read_Buffer(PORT,Receive,6);                       // Read th PORT

    //Vérifie si ce qui a été reçu correspond à ce qui a été envoyé si non, renvoie -1
    // Check if the string received corresponding to the string sent
    if(Receive[0]!=0x1B) return -1;
    if(Receive[1]!=0x4C) return -1;
    if(Receive[2]!=0x4F) return -1;
    if(Receive[3]!=0x3A) return -1;
    if(Receive[4]!=0x0D) return -1;
    if(Receive[5]!=0x1A) return -1;
    return 0;
}



//************************* Allow to change mode to mode 1*************************//

unsigned char CNC_START_DCC_MODE(int PORT)
{
    unsigned char Send[6]={0x1B,'L','I',':',0x0D,0x1A};      // ESC L I : CR SUB
    unsigned char Receive[6];
    while(!(UART_GET_CTS(PORT)));                               // Wait the device isn't busy
    UART_Send_Buffer (PORT,Send,6);                             // Send the table
    UART_Read_Buffer(PORT,Receive,6);                           // read the PORT

    // Check if the string received corresponding to the string sent
    if(Receive[0]!=0x1B) return -1;
    if(Receive[1]!=0x4C) return -1;
    if(Receive[2]!=0x49) return -1;
    if(Receive[3]!=0x3A) return -1;
    if(Receive[4]!=0x0D) return -1;
    if(Receive[5]!=0x1A) return -1;
    return 0;
}


//************************ Allow to change mode to mode 2************************//

unsigned char CNC_START_HPGL_LINE_MODE(int PORT)
{
    unsigned char Send[6]={0x1B,'L','M',':',0x0D,0x1A};          // ESC L M : CR SUB
    unsigned char Receive[6];
    while(!(UART_GET_CTS(PORT)));                                   // Wait the device isn't busy
    UART_Send_Buffer (PORT,Send,6);                                 // Send the table
    UART_Read_Buffer(PORT,Receive,6);                               // Read the port

    // Check if the string received corresponding to the string sent
    if(Receive[0]!=0x1B) return -1;
    if(Receive[1]!=0x4C) return -1;
    if(Receive[2]!=0x4D) return -1;
    if(Receive[3]!=0x3A) return -1;
    if(Receive[4]!=0x0D) return -1;
    if(Receive[5]!=0x1A) return -1;
    return(0);
}

//************************ Allow to change mode to mode 3***********************//

unsigned char CNC_START_HPGL_FILE_MODE(int PORT)
{
    unsigned char Send[6]={0x1B,'F','M',':',0x0D,0x1A};          // ESC F M : CR SUB
    unsigned char Receive[6];
    while(!(UART_GET_CTS(PORT)));                                   // Wait the device isn't busy
    UART_Send_Buffer (PORT,Send,6);                                 // Send the table
    UART_Read_Buffer(PORT,Receive,6);                               // Read the port


    // Check if the string received corresponding to the string sent
    if(Receive[0]!=0x1B) return -1;
    if(Receive[1]!=0x46) return -1;
    if(Receive[2]!=0x4C) return -1;
    if(Receive[3]!=0x3A) return -1;
    if(Receive[4]!=0x0D) return -1;
    if(Receive[5]!=0x1A) return -1;
    return(0);
}

//********************** Return the actual mode**************************//

unsigned char CNC_OUTPUT_CURRENT_MODE(int PORT)
{
    unsigned char Send[6]={0x1B,'M','D',':',0x0D,0x1A};          // ESC M D : CR SUB
    unsigned char Receive[7];
    while(!(UART_GET_CTS(PORT)));                                   // Wait the device isn't busy
    UART_Send_Buffer (PORT,Send,6);                                 // Send the table
    UART_Read_Buffer(PORT,Receive,7);                               // Read the port


     // Check if the string received corresponding to the string sent
    if(Receive[0]!=0x1B) return -1;
    if(Receive[1]!=0x4D) return -1;
    if(Receive[2]!=0x44) return -1;
    if(Receive[3]!=0x3A) return -1;
    if(Receive[5]!=0x0D) return -1;
    if(Receive[6]!=0x1A) return -1;
    return Receive[4]-'0'; // Retourne le mode
}

//************************ Exit HPGL mode**********************//

unsigned char CNC_HPGL_PLOTTER_OFF(int PORT)
{
    unsigned char Send[5]={0x1B,'.',')',0x0D,0x1A};
    while(!(UART_GET_CTS(PORT)));                               // Wait the CTS bit change to 0
    UART_Send_Buffer (PORT,Send,5);                             // Send the table
    while((UART_GET_CTS(PORT)));                                // Wait the CTS bit change to 1
    return 0;
}


//*******************************Mode Change**********************//

unsigned char CNC_CHANGE_MODE(int PORT,int MODE)
{
    while(!(UART_Empty_Data_Holding_Register(PORT)));   // Wait the buffer is empty

    switch (CNC_OUTPUT_CURRENT_MODE(PORT))              // Check the actual mode
    {
        case 0 :        // if mode 0
                switch(MODE)    // desired mode
                {
                    case 1 : CNC_START_DCC_MODE(PORT);break; // if 1, then change mode to 1
                    case 2 : CNC_START_HPGL_LINE_MODE(PORT);break;  // if 2, then change mode to 2
                    case 3 : CNC_START_HPGL_FILE_MODE(PORT);break;  // if 3, then change mode to 3
                    default :break;
                }
                return 0;
                break;

        case 1 :    // if mode 1
                switch(MODE)    // desired mode
                {
                    case 0 : CNC_RETURN_LOCAL_MODE(PORT);break; // if 0, then change mode to 0
                    case 2 : CNC_START_HPGL_LINE_MODE(PORT);break;  // if 2, then change mode to 2
                    case 3 : CNC_START_HPGL_FILE_MODE(PORT);break;  // if 3, then change mode to 3
                    default :break;
                }
                return 0;
                break;

        case 2 :    // if mode 2
                switch(MODE)    // desired mode
                {
                    case 0 :  CNC_HPGL_PLOTTER_OFF(PORT);  break;   // if 0, then change mode to 0
                    case 1 : CNC_HPGL_PLOTTER_OFF(PORT);CNC_START_DCC_MODE(PORT);break; // if 1, then change mode to 0 before change to mode 1
                    case 3 : CNC_HPGL_PLOTTER_OFF(PORT);CNC_START_HPGL_FILE_MODE(PORT);break;   // if 3, then change mode to 0 before change to mode 3
                    default :break;
                }
                return 0;
                break;

        case 3 :    // if mode 3
                switch(MODE)    // desired mode
                {
                    case 0 : CNC_HPGL_PLOTTER_OFF(PORT);break;  // if 0, then change mode to 0
                    case 1 : CNC_HPGL_PLOTTER_OFF(PORT);CNC_START_DCC_MODE(PORT);break; // if 1, then change mode to 0 before change to mode 1
                    case 2 : CNC_HPGL_PLOTTER_OFF(PORT);CNC_START_HPGL_LINE_MODE(PORT);break;   // if 2, then change mode to 0 before change to mode 2
                    default :break;
                }
                return 0;
                break;
    }

    return 0;

}


