
/*************************************************************************
------------------C# FILE---------------------------------------------
------This File contains differents functions which allow to communicate--
------with the device in order to move it---------------------------------
------They functions is a reunion of differents HPGL instructions that we-
------found in the manufacturer documentation.----------------------------
-------------------------------------by AUDUREAU YOAN---------------------
-------------------------------------& DURAND ANTOINE---------------------
*************************************************************************/

#include "CNC_MODE.h"
#include "CNC.h"
#include "UART.h"
#include "CNC_MOVE.h"
#include <string.h>


//landmarks for X,Y and Z
#define POS_X_MAX 11999
#define POS_Y_MAX 15999
#define POS_X_MIN 0
#define POS_Y_MIN 0
#define POS_Z_MAX 2999
#define POS_Z_MIN 0




unsigned char CNC_VELOCITY_SELECT(int PORT,unsigned char vs)
{
    int A,B;
    A=((vs/10)%10)+'0'; // A contain the decimals of vs
    B=(vs%10)+'0';      // B contain the units of vs
    unsigned char Send[10]={0x56,0x53,0x20,0x36,0x30,0x2E,0x30,0x3B,0x0D,0x1A};
    // Changing the table to redefine the speed
    Send[3]=A;
    Send[4]=B;
    while(!(UART_GET_CTS(PORT)));   // wait the bit CTS changes to 0 (wait the device is not busy)
    UART_Send_Buffer (PORT,Send,10);    // Send the table
    while((UART_GET_CTS(PORT)));    // wait the bit CTS changes to 1
    return 0;
}
// if 0 returned ==> OK
// if -1 returned ==> coordinates X too large
// if -2 returned ==> coordinates Y too large



//////////////////////////////////Function ablolute move/////////////////////////////////

unsigned char CNC_PLOT_ABSOLUTE(int PORT,int PosX,int PosY)
{
    unsigned char Send[20]={0x50,0x41,0x20,0x30,0x30,0x30,0x30,0x30,0x30,0x2C,0x20,0x30,0x30,0x30,0x30,0x30,0x30,0x3B,0x0D,0x1A};   //table to send

    if((PosX<=POS_X_MAX)&&(PosX>=POS_X_MIN))    // if the device is less than POS_X_MAX and highter than POS_X_MIN
    {
        Send[4]= (PosX/10000)%10+'0';   //Fill the table "Send" with the variable PosX
        Send[5]= (PosX/1000)%10+'0';
        Send[6]= (PosX/100)%10+'0';
        Send[7]= (PosX/10)%10+'0';
        Send[8]= (PosX%10)+'0';
    }
    else return -1; // else exit

    if((PosY<=POS_Y_MAX)&&(PosY>=POS_Y_MIN))    // if the device is less than POS_Y_MAX and highter than POS_Y_MIN
    {
        Send[12]= (PosY/10000)%10+'0';  //Fill the table "Send" with the variable PosY
        Send[13]= (PosY/1000)%10+'0';
        Send[14]= (PosY/100)%10+'0';
        Send[15]= (PosY/10)%10+'0';
        Send[16]= (PosY%10)+'0';
    }
    else return -2; // else exit

    UART_Send_Buffer (PORT,Send,20);    // Send the table

    return 0;
}

///////////////////////////Déplacement relatif///////////////////////////

unsigned char CNC_PLOT_RELATIVE(int PORT,int *X,int *Y,int *Z,int PosX,int PosY)
{
   unsigned char Send[20]={'P','R',0x20,'0','0','0','0','0','0',',',0x20,'0','0','0','0','0','0',';',0x0D,0x1A};    //Table to send
    CNC_CHANGE_MODE(PORT, MODE_2);  // Transition to mode 2
    CNC_OUTPUT_COMMANDED_POSITION(PORT, &X, &Y, &Z);    // Receive the actual position
    UART_Empty_Data_Holding_Register(PORT);             //wait that this register is empty
    int x=X,y=Y,a=0,b=0;
    a=PosX;
    b=PosY;
    //printf("x:%d y:%d PosX:%d PosY:%d a:%d b:%d\n\n",x,y,PosX,PosY,a,b);
    if((PosX>=0)&&((x+PosX)<POS_X_MAX))
        {
            Send[4]= (PosX/10000)%10+'0';   //Fill the table "Send" with the variable PosX
            Send[5]= (PosX/1000)%10+'0';
            Send[6]= (PosX/100)%10+'0';
            Send[7]= (PosX/10)%10+'0';
            Send[8]= (PosX%10)+'0';
        }

    if((a<0)&&((x+a)>POS_X_MIN))
        {
            a=-a;
            Send[3]='-';
            Send[4]= (a/10000)%10+'0';  //Fill the table "Send" with the variable PosX
            Send[5]= (a/1000)%10+'0';
            Send[6]= (a/100)%10+'0';
            Send[7]= (a/10)%10+'0';
            Send[8]= (a%10)+'0';
        }

    if((PosY>=0)&&((y+PosY)<POS_Y_MAX))
        {
            Send[12]= (PosY/10000)%10+'0';  //Fill the table "Send" with the variable PosY
            Send[13]= (PosY/1000)%10+'0';
            Send[14]= (PosY/100)%10+'0';
            Send[15]= (PosY/10)%10+'0';
            Send[16]= (PosY%10)+'0';
        }

    if((b<0)&&((y+b)>POS_Y_MIN))
        {
            b=-b;
            Send[11]='-';
            Send[12]= (b/10000)%10+'0'; //Fill the table "Send" with the variable PosY
            Send[13]= (b/1000)%10+'0';
            Send[14]= (b/100)%10+'0';
            Send[15]= (b/10)%10+'0';
            Send[16]= (b%10)+'0';
        }
    UART_Send_Buffer (PORT,Send,20);    // Send the table
    CNC_CHANGE_MODE(PORT, MODE_2);      // transition to mode 2
    CNC_OUTPUT_COMMANDED_POSITION(PORT, &X, &Y, &Z);    // Receive The actual position of the device

    return 0;
}



void CNC_Init_XYZ(int PORT)
{
    unsigned char Send[6]={'@','H','M',';',0x0D,0x1A};
    UART_Empty_Data_Holding_Register(PORT); // wait that this register is empty
    UART_Send_Buffer (PORT,Send,6);         // Send the table
}



unsigned char CNC_MOVE_XY(int PORT,int *X,int *Y,int *Z,unsigned char vs,int PosX,int PosY)
{
    unsigned char ErrorPos=0;
    CNC_VELOCITY_SELECT(PORT,vs);                               // Speed

    CNC_CHANGE_MODE(PORT, MODE_2);                              // Transition to mode 2

    CNC_OUTPUT_COMMANDED_POSITION(PORT,&X,&Y,&Z);               // Check the device's position

    UART_Empty_Data_Holding_Register(PORT);                     // wait that this register is empty

    ErrorPos=CNC_PLOT_ABSOLUTE(PORT,PosX,PosY);                 // XY move
    if(ErrorPos) return ErrorPos;                               // if erreur exit
    CNC_CHANGE_MODE(PORT, MODE_2);                              // Transition to mode 2

    CNC_OUTPUT_ABSOLUTE_COMMANDED_POSITION(PORT, &X, &Y, &Z );  // Check the device's position

    return ErrorPos; //exit
}






unsigned char CNC_VELOCITY_Z(int PORT,unsigned char zd)
{

    int A,B;
    A=((zd/10)%10)+'0'; // A contain the zd decimals
    B=(zd%10)+'0';      // B contain the zd units
    unsigned char Send[12]={'@','Z','D',0x20,'6','0','.','0',';',0x0D,0x1A};
    Send[4]=A;
    Send[5]=B;
    while(!(UART_GET_CTS(PORT)));       // Wait the bit CTS changes to 0
    UART_Send_Buffer (PORT,Send,12);    // Send the table
    while((UART_GET_CTS(PORT)));        // Wait the bit CTS changes to 1
    return 0;
}



unsigned char CNC_ZAXIS_RELATIVE_MOVE(int PORT,int *X,int *Y,int *Z,int PosZ)
{

    unsigned char Send[11]={'@','Z','D',0x20,'0','0','0','0','0',';',0x0D,0x1A};
    CNC_OUTPUT_COMMANDED_POSITION(PORT, &X, &Y, &Z);    // receive the actual device's position

int z=Z;


    if((PosZ>=0)&&((z+PosZ)<POS_Z_MAX))
    {
        Send[5]= (PosZ/1000)%10+'0';    // Fill the table with PosZ
        Send[6]= (PosZ/100)%10+'0';
        Send[7]= (PosZ/10)%10+'0';
        Send[8]= (PosZ%10)+'0';
    }

     if((PosZ<0)&&((z+PosZ)>POS_Z_MIN))
    {
        printf("ok");
        PosZ=-PosZ;
        Send[4]='-';                    // Fill the table with "-"
        Send[5]= (PosZ/1000)%10+'0';    // Fill the table with PosZ
        Send[6]= (PosZ/100)%10+'0';
        Send[7]= (PosZ/10)%10+'0';
        Send[8]= (PosZ%10)+'0';
    }



    while(!(UART_GET_CTS(PORT)));         // Wait the bit CTS changes to 0

    UART_Send_Buffer (PORT,Send,11);      //Send the table
        CNC_CHANGE_MODE(PORT, MODE_2);
    CNC_OUTPUT_COMMANDED_POSITION(PORT, &X, &Y, &Z);    // receive the actual device's position
    while((UART_GET_CTS(PORT)));    // Wait the bit CTS changes to 1
    return 0;
}



unsigned char CNC_ZAXIS_ABSOLUTE_MOVE(int PORT,int PosZ)
{

    unsigned char Send[11]={'@','Z','A',0x20,'0','0','0','0',';',0x0D,0x1A};    //table to send

    if(PosZ<=POS_Z_MAX)
    {
        Send[4]= (PosZ/1000)%10+'0';    // Fill the table with PosZ
        Send[5]= (PosZ/100)%10+'0';
        Send[6]= (PosZ/10)%10+'0';
        Send[7]= (PosZ%10)+'0';
    }
    else return 1;  //else exit

    while(!(UART_GET_CTS(PORT)));   // Wait the bit CTS changes to 0

    UART_Send_Buffer (PORT,Send,11);  // transition to mode 2
    while((UART_GET_CTS(PORT)));    // Wait the bit CTS changes to 1
    return 0;
}


unsigned char CNC_MOVE_Z(int PORT,int *X,int *Y, int *Z,unsigned char zd,int PosZ)
{

    unsigned char ErrorPos=0;
    CNC_VELOCITY_Z(PORT,zd);                                        //zd=speed on Z
    CNC_CHANGE_MODE(PORT, MODE_2);                                  //transition to mode 2
    CNC_OUTPUT_COMMANDED_POSITION(PORT,&X,&Y,&Z);                   //return the position
    ErrorPos=CNC_ZAXIS_ABSOLUTE_MOVE(PORT,PosZ);                    //Move
    if(ErrorPos)    return ErrorPos;                                //if z>3000 exit

    CNC_CHANGE_MODE(PORT, MODE_2);                                  // transition to mode 2
    CNC_OUTPUT_ABSOLUTE_COMMANDED_POSITION(PORT, &X, &Y, &Z );      // receive the actual device position

    return ErrorPos;
}



