/**------------------C# FILE--------------------------------------------
------This File contains differents functions that allow to configure------
------the serial port.----------------------------------------------------
------This feature also allows the reading of serial port in order to-----
------detect state changes.-----------------------------------------------
-------------------------------------by AUDUREAU YOAN---------------------
-------------------------------------& DURAND ANTOINE---------------------
**************************************************************************
-----RTS : Request To Send------------------------------------------------
-----DTR : Data Terminal Ready--------------------------------------------
-----CTS : lear To Send---------------------------------------------------
*************************************************************************/


#include <stdio.h>
#include <sys/io.h>
#include "UART.h"


char Open_Port(int PORT)
{
    //printf("%d",ioperm(PORT,8,1));  //Displays the ioperm return value
   	return (ioperm(PORT,8,1));      //return -1 if success or 0 if not
}




void Config_Port (int PORT, int Baud, unsigned char Parity, unsigned char Stop_bit, unsigned char lenght)
{
    outb(0, PORT + 1 );     //Disable interruptions
	outb(0x80, PORT + 3 );  //Enable DLAB
	outb(Baud, PORT + 0 );  // Set Baud rate - Divisor Latch Low Byte
	outb( (Baud) >> 8 , PORT + 1 );    // Set Baud rate - Divisor Latch High Byte
	outb((Parity|Stop_bit|lenght) , PORT + 3 );  // DBAL OFF - 8 Bits, No Parity, 1 Stop Bit
	outb(0xC7 , PORT + 2 );  // FIFO Control Register
    outb(0x00, PORT + 4 );   // Turn on DTR, RTS, and OUT2
                            /*  Bit 7 Reserved
                                Bit 6 Reserved
                                Bit 5 Autoflow Control Enabled (16750 only)
                                Bit 4 LoopBack Mode
                                Bit 3 Aux Output 2
                                Bit 2 Aux Output 1
                                Bit 1 Force Request to Send
                                Bit 0 Force Data Terminal Ready */
}

void Set_RTS (int PORT)
{
    unsigned char A=inb(PORT+4);    //Read the  register PORT+4
    outb(A|0x02,PORT+4);            //Set the RTS (bit n°1) to 1
}

void Reset_RTS (int PORT)
{
    unsigned char A=inb(PORT+4);    //Read the  register PORT+4
    outb(A&0xFD,PORT+4);            //Set the RTS (bit n°1) to 0
}

void Set_DTR (int PORT)
{
    unsigned char A=inb(PORT+4);    //Read the  register PORT+4
    outb(A|0x01,PORT+4);            //Set the DTR (bit n°0) to 1
}

void Reset_DTR (int PORT)
{
    unsigned char A=inb(PORT+4);    //Read the  register PORT+4
    outb((A&0xFE),PORT+4);          //Set the DTR bit (bit n°0) to 0
}



unsigned char UART_GET_CTS(int PORT)
{
    unsigned char A=inb(PORT+6);    //Read the  register PORT+6
    return (A & 0x10);              //Return the value of CTS (bit n°4)
}

unsigned char UART_GET_RTS(int PORT)
{

    unsigned char A=inb(PORT+4);    //Read the  register PORT+4
    return (A & 0x02);              //Return the value of RTS (bit n°1)
    }

char Transmit(int PORT, unsigned char val)
{
    outb( val , PORT); 	     //Send Character to Serial Port
}


char Receive (int PORT)
{
    return inb (PORT);      //Receive character
}


char Is_Data_Ready(int PORT)
{
    return inb(PORT+5) & 0x01;  //Return the value of bit n°0 of register 5
}

void UART_Send_Buffer (int PORT, char Buffer[], int Nb)
{
    int i;
    // printf ("         Send -> ");
    for(i=0;i<Nb;i++)               // for the first Character in the table "buffer" at last
    {
        Transmit(PORT,Buffer[i]);   // Send the character corresponding to the box i
        //printf("%c ",Buffer[i]);
    }
    printf ("\n");
}



void UART_Read_Buffer(int PORT, unsigned char Receive[], int Nb)
{
    int i;
    //printf ("        Receive -> ");
    for(i=0;i<Nb;i++)                   // for the first Character in the table "Receive" at last
    {
        while(!(Is_Data_Ready(PORT)));  // while the device is not ready, wait
        Receive[i]=inb(PORT);           // the  i box in the table "Receive" take the value of PORT
        //printf("%c ",Receive[i]);
    }
    //printf ("\n");
}


char UART_Empty_Data_Holding_Register(int PORT)
{
    return inb(PORT+5) & 0x40;      //Return the value of bit 4 in port+5
}


